/*! \file Beam.h 
 *  \brief: header file for beam object
 */

#ifndef _BEAM_H_
#define _BEAM_H_


#include "./Element.h"
#include "./Node.h"
#include "./Matice.h"
#include "./Matpar.h"
#include "../ModelProcessorx/IoModel.h"
#include "./Hook.h"

class Object;
class DataSet;
class Element;
class Hook;

class Beam: public Element{

	private: 

		/*ids:*/
		int id;

		Hook hnodes;  //hook to 2 nodes
		Hook hmatice; //hook to 1 matice
		Hook hmatpar; //hook to 1 matpar
		Hook hnumpar; //hook to 1 numpar

		Inputs* inputs;
	
	public:

		/*constructors, destructors: {{{1*/
		Beam();
		Beam(int beam_id,int* beam_node_ids, int beam_matice_id, int beam_matpar_id, int beam_numpar_id);
		Beam(int beam_id,Hook* beam_hnodes, Hook* beam_hmatice, Hook* beam_hmatpar, Hook* beam_hnumpar, Inputs* beam_inputs);
		Beam(int i, IoModel* iomodel);
		~Beam();
		/*}}}*/
		/*object management: {{{1*/
		void  Echo();
		void  DeepEcho();
		void  Marshall(char** pmarshalled_dataset);
		int   MarshallSize();
		char* GetName();
		void  Demarshall(char** pmarshalled_dataset);
		int   Enum();
		int   GetId(); 
		int   MyRank();
		void  Configure(void* loads,void* nodes,void* materials,void* parameters);
		Object* copy();
		void  SetClone(int* minranks);

		/*}}}*/
		/*numerics: {{{1*/
		void  CreateKMatrix(Mat Kgg,int analysis_type,int sub_analysis_type);
		void  CreatePVector(Vec pg,  int analysis_type,int sub_analysis_type);
		void  UpdateInputs(double* solution, int analysis_type, int sub_analysis_type);
		void  GetDofList(int* doflist,int* pnumberofdofs);
		void  GetDofList1(int* doflist);
		void  CreateKMatrixDiagnosticHutter(Mat Kgg,int analysis_type,int sub_analysis_type);
		void  GetParameterValue(double* pp, double* plist, double* gauss_l1l2l3);
		void  CreatePVectorDiagnosticHutter(Vec pg,int analysis_type,int sub_analysis_type);
		void* GetMatPar();

		void  ComputeBasalStress(Vec sigma_b,int analysis_type,int sub_analysis_type);
		void  ComputePressure(Vec p_g,int analysis_type,int sub_analysis_type);
		void  ComputeStrainRate(Vec eps,int analysis_type,int sub_analysis_type);
		void  GetNodes(void** vpnodes);
		/*}}}*/
		/*not implemented: {{{1*/
		bool  GetShelf();
		bool  GetOnBed();
		void  GetBedList(double*);
		void  GetThicknessList(double* thickness_list);
		void  Du(_p_Vec*,void*, int,int);
		void  Gradj(_p_Vec*, void*, int, int,char*);
		void  GradjDrag(_p_Vec*, void*, int,int );
		void  GradjB(_p_Vec*, void*, int,int );
		double Misfit(void*,int,int);
		double SurfaceArea(void*,int,int);
		double CostFunction(void*,int,int);
		void  GetNodalFunctions(double* l1l2, double gauss_coord);
		void  GetParameterValue(double* pvalue, double* value_list,double gauss_coord);
		void  GetJacobianDeterminant(double* pJdet,double* z_list, double gauss_coord);
		double MassFlux(double* segment,double* ug);
		/*}}}*/

};
#endif  /* _BEAM_H */
